@@ -1,6 +1,38 @@
 from collections import defaultdict
-from exceptions import NotImplementedError
-import mysql.connector as mysql
+import re
+
+import pymysql as mysql
+
+
+MYSQL_FIELD_TYPES = {
+    0: 'DECIMAL',
+    1: 'TINY',
+    2: 'SHORT',
+    3: 'LONG',
+    4: 'FLOAT',
+    5: 'DOUBLE',
+    6: 'NULL',
+    7: 'TIMESTAMP',
+    8: 'LONGLONG',
+    9: 'INT24',
+    10: 'DATE',
+    11: 'TIME',
+    12: 'DATETIME',
+    13: 'YEAR',
+    14: 'NEWDATE',
+    15: 'VARCHAR',
+    16: 'BIT',
+    246: 'NEWDECIMAL',
+    247: 'INTERVAL',
+    248: 'SET',
+    249: 'TINY_BLOB',
+    250: 'MEDIUM_BLOB',
+    251: 'LONG_BLOB',
+    252: 'BLOB',
+    253: 'VAR_STRING',
+    254: 'STRING',
+    255: 'GEOMETRY'
+}
 
 
 class DataSource:
@@ -199,24 +231,25 @@ class MySQLDataSource(DataSource):
 
     def fetch_types(self, table, cols):
         with self.connect() as con:
-            cursor = con.cursor() 
+            cursor = con.cursor()
             cursor.execute("SELECT %s FROM `%s` LIMIT 1" % (self.fmt_cols(cols), table))
             cursor.fetchall()
             types = {}
             for desc in cursor.description:
-                types[desc[0]] = mysql.FieldType.get_info(desc[1])
+                # types[desc[0]] = mysql.FieldType.get_info(desc[1])
+                types[desc[0]] = MYSQL_FIELD_TYPES[desc[1]]
         return types
 
     def fetch(self, table, cols):
         with self.connect() as con:
-            cursor = con.cursor() 
+            cursor = con.cursor()
             cursor.execute("SELECT %s FROM %s" % (self.fmt_cols(cols), table))
             result = [cols for cols in cursor]
         return result
 
     def select_where(self, table, cols, pk_att, pk):
         with self.connect() as con:
-            cursor = con.cursor() 
+            cursor = con.cursor()
             attributes = self.fmt_cols(cols)
             cursor.execute("SELECT %s FROM %s WHERE `%s`='%s'" % (attributes, table, pk_att, pk))
             result = [cols for cols in cursor]
@@ -266,7 +299,7 @@ class PgSQLDataSource(DataSource):
                       WHERE constraint_type = 'FOREIGN KEY' AND tc.table_catalog='%s'" % database)
             fk_result = [row for row in cursor]
         return fk_result
-        
+
     def table_column_names(self):
         with self.connect() as con:
             cursor = con.cursor()
@@ -278,7 +311,7 @@ class PgSQLDataSource(DataSource):
                 information_schema.table_constraints AS tc\
                 JOIN information_schema.key_column_usage AS kcu \
                 ON tc.constraint_name = kcu.constraint_name \
-                WHERE constraint_type = 'PRIMARY KEY' AND tc.table_catalog='%s'" % database) 
+                WHERE constraint_type = 'PRIMARY KEY' AND tc.table_catalog='%s'" % database)
             tbl_col_names = [row for row in cursor]
         return tbl_col_names
 
@@ -302,7 +335,7 @@ class PgSQLDataSource(DataSource):
 
     def fmt_cols(self, cols):
         return ','.join(["%s" % col for col in cols])
-    
+
     def fetch_types(self, table, cols):
         with self.connect() as con:
             cursor = con.cursor()
@@ -316,14 +349,14 @@ class PgSQLDataSource(DataSource):
 
     def fetch(self, table, cols):
         with self.connect() as con:
-            cursor = con.cursor() 
+            cursor = con.cursor()
             cursor.execute("SELECT %s FROM %s" % (self.fmt_cols(cols), table))
             result = [cols for cols in cursor]
         return result
 
     def select_where(self, table, cols, pk_att, pk):
         with self.connect() as con:
-            cursor = con.cursor() 
+            cursor = con.cursor()
             attributes = self.fmt_cols(cols)
             cursor.execute("SELECT %s FROM %s WHERE %s='%s'" % (attributes, table, att, val_att))
             result = [cols for cols in cursor]
@@ -335,9 +368,103 @@ class PgSQLDataSource(DataSource):
             cursor.execute("SELECT DISTINCT %s, %s FROM %s" % (col, col, table))
             values = [val for (_,val) in cursor]
         return values
-    
+
     def get_driver_name(self):
         return 'org.postgresql.Driver'
 
     def get_jdbc_prefix(self):
         return 'jdbc:postgresql://'
+
+
+class SQLiteDataSource(DataSource):
+    '''
+    A DataSource implementation for accessing datasets from SQLite DBMS.
+    Because SQLite types are dynamic and its CREATE TABLE statement allows for any kind of custom type names, we try to match the most common types with three type categories used in ``rdm``.
+    '''
+    continuous_types = ('real', 'double', 'double precision', 'float', 'decimal')
+    integer_types = ('int', 'integer', 'tinyint', 'smallint', 'mediumint', 'bigint', 'unsigned big int', 'int2', 'int8')
+    ordinal_types = ('character', 'varchar', 'varying character', 'nchar', 'native character', 'nvarchar', 'text', 'clob', 'char')
+
+    def __init__(self, connection):
+        '''
+            :param connection: a DBConnection instance.
+        '''
+        self.connection = connection
+
+    def connect(self):
+        return self.connection.connect()
+
+    def foreign_keys(self):
+        tables = self.tables()
+        result = []
+        for table in tables:
+            with self.connect() as con:
+                cursor = con.cursor()
+                cursor.execute('''SELECT "from","table","to" FROM pragma_foreign_key_list("{}");'''.format(table))
+                for row in cursor:
+                    result.append((table,) + row)
+        return result
+
+    def table_column_names(self):
+        tables = self.tables()
+        result = []
+        for table in tables:
+            with self.connect() as con:
+                cursor = con.cursor()
+                cursor.execute('SELECT name FROM pragma_table_info("{}") WHERE pk=1;'.format(table))
+                result.append((table, [row for row in cursor][0][0]))
+        return result
+
+    def tables(self):
+        with self.connect() as con:
+            cursor = con.cursor()
+            cursor.execute("SELECT name FROM sqlite_master \
+                            WHERE type='table' AND name NOT LIKE 'sqlite_%';")
+            tables = [table for (table,) in cursor]
+        return tables
+
+    def table_columns(self, table_name):
+        with self.connect() as con:
+            cursor = con.cursor()
+            cursor.execute('SELECT name FROM pragma_table_info("{}");'.format(table_name))
+            columns = [col for (col,) in cursor]
+        return columns
+
+    def fmt_cols(self, cols):
+        return ','.join(['"{}"'.format(col) for col in cols])
+
+    def fetch_types(self, table, cols):
+        types = {}
+        with self.connect() as con:
+            cursor = con.cursor()
+            cursor.execute('SELECT "name", "type" from pragma_table_info("{}") WHERE "name" IN ({})'.format(table, self.fmt_cols(cols)))
+            types = {row[0]: re.sub('\([0-9a-zA-Z ,]*\)', '', row[1].strip().lower()).strip() for row in cursor}
+        return types
+
+    def fetch(self, table, cols):
+        with self.connect() as con:
+            cursor = con.cursor()
+            cursor.execute('SELECT %s FROM "%s"' % (self.fmt_cols(cols), table))
+            result = [cols for cols in cursor]
+        return result
+
+    def select_where(self, table, cols, pk_att, pk):
+        with self.connect() as con:
+            cursor = con.cursor()
+            attributes = self.fmt_cols(cols)
+            cursor.execute('SELECT %s FROM "%s" WHERE "%s"="%s"' % (attributes, table, pk_att, pk))
+            result = [cols for cols in cursor]
+        return result
+
+    def column_values(self, table, col):
+        with self.connect() as con:
+            cursor = con.cursor()
+            cursor.execute('SELECT DISTINCT "%s" FROM "%s"' % (col, table))
+            values = [val[0] for val in cursor]
+        return values
+
+    def get_driver_name(self):
+        return 'java.sql.DriverManager'
+
+    def get_jdbc_prefix(self):
+        return 'jdbc:sqlite:{}'.format(self.connection.sqlite_database)
